/***************************************************************************
                          fileoperation.cpp  -  description
                             -------------------
    begin                : Sun Nov 11 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kapplication.h>
#include <klocale.h>
#include <kio/netaccess.h>

// QT includes
#include <qeventloop.h>
#include <qfileinfo.h>

// OS includes
#include <stdio.h>
#include <unistd.h>

// Own includes
#include "fileoperation.h"
#include "ProgressDialog.h"
#include "batchrenamer.h"

using namespace KIO;

FileOperation::FileOperation()
{
    canceled = false;
}

FileOperation::~FileOperation() { }

bool FileOperation::start( KURL src, KURL dest, int mode, bool overwrite )
{
    locked = true;
    result = 0;

    // check for file overwriting
    // might slow things down!
    // find a better solution
    if( NetAccess::exists( dest ) ) {
        if( overwrite ) {
            // TODO: do not delete the file
            // tell copy job to just overwrite
            // maybe we need an own subclass
            if( src != dest && !NetAccess::del( dest ) ) {
                m_error = NetAccess::lastErrorString();
            //TODO: warning message!
            // p->warning( QString( i18n("Overwriting file %1.") ).arg( dst ));

                return false;
            }
        } else {
            m_error =  QString( i18n("Output file %1 exists already.") ).arg( dest.prettyURL() );
            return false;
        }
    }

    KURL::List lsrc;
    lsrc.append( KURL(src) );

    switch( mode ) {
        case RENAME:
        case MOVE:
        default:
            mode = CopyJob::Move; break;
        case COPY:
            mode = CopyJob::Copy; break;
        case LINK:
            mode = CopyJob::Link; break;
    };

    CopyJob* job = new CopyJob( lsrc, KURL(dest),
                        (CopyJob::CopyMode)mode, true, false );
    job->setAutoErrorHandlingEnabled( true, 0 );
    connect( job, SIGNAL( result (KIO::Job *) ),
           this, SLOT( slotResult (KIO::Job *) ) );

    kapp->eventLoop()->enterLoop();
    return !result;
}

bool FileOperation::fcopy( QString src, QString dest )
{
    return start( KURL( src ), KURL( dest ), KIO::CopyJob::Copy, false );
}

QString FileOperation::getName( QString file )
{
    QFileInfo info( file );
    return info.fileName();
}

void FileOperation::slotResult( KIO::Job * job )
{
    result = job->error();
    if( result ) {
        m_error = job->errorString();
        qDebug("ERROR:" + m_error );
    }

    kapp->eventLoop()->exitLoop();
}

