/***************************************************************************
                          kmylistbox.h  -  description
                             -------------------
    begin                : Tue Oct 16 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMYLISTBOX_H
#define KMYLISTBOX_H

// KDE includes
#include <klistbox.h>
#include <kurl.h>
#include <kfileitem.h>

// Qt includes
#include <qstringlist.h>

class KRecursiveLister;
class KDirLister;
class KFileItem;
class QDragObject;
class QPixmap;
class QPoint;
class KMyListBox : public KListBox  {
    Q_OBJECT
    public:
        KMyListBox(QWidget* parent=0, const char* name=0, WFlags fl=0);
        ~KMyListBox();

        // call before you add something
        void startAdding();
        // call after you added something
        void endAdding();

        void addFile( KURL filename, bool isfile = false );
        void addDirName( KURL dirname );
        
        void addDir( KURL dirname, const QString & m_filter, bool m_hidden, bool recursively, bool dirnames = false );
        void addDirName( KURL dirname, const QString & m_filter, bool m_hidden, bool recursive = false );
        void setPreview( bool prv );


        void setName( bool name );

        void move( int i );
        void setPreviewSize( int size );
        int getPreviewSize() const { return previewSize; }

        KURL url( int index ) const;
        bool dir( int index ) const;

    public slots:
        void moveMode();
        void moveDown();
        void moveDown( int i );
        void moveUp();
        void moveUp( int i );

        void sortAscending();
        void sortDescending();

    signals:
        // emited when new item is drag'n'dropped into listobox
        // or when files are removed or something similar
        void updateCount();
        void updatePreview();
        void deletePressed();

    private:
        void preview( KURL::List list );

        void dropEvent( QDropEvent* e );
        void dragEnterEvent( QDragEnterEvent* e );
        void viewportMousePressEvent( QMouseEvent* e );
        void viewportMouseReleaseEvent( QMouseEvent* e );
        void viewportMouseMoveEvent( QMouseEvent* e );
        void keyPressEvent( QKeyEvent* e );
        void keyReleaseEvent( QKeyEvent* e );

        bool isFile( KURL f );
        bool isInList( KURL text );

        void qSortList( KFileItem* first, KFileItem* last );
        void sortList( KFileItemList* list, KFileItem* first, KFileItem* last );
        inline int cmpItem( KFileItem* item1, KFileItem* item2 );

    private slots:
        void openFile( QListBoxItem* item );
        void select( QListBoxItem* item );

        void previewDone( const KFileItem* item, const QPixmap &pixmap );
        void previewFailed( const KFileItem* item );
        void previewFinished();

        void listerFinished();
        void listerFinishedRecursivelyDirnames();
        void listerFinishedRecursivelyNoDirnames();
        void listerFinishedRecursively( bool dirnames );
        void nameListerFinished();

    protected:
        bool drag;
        bool mousePressed;
        bool ctrlPressed;
        bool shiftPressed;
        bool moving;
        int previewSize;

        KDirLister* lister;
        KRecursiveLister* reclister;

        QDragObject* drobj;
        QPoint presspos;
};

class KMyListBoxItem : public QListBoxItem {
    public:
        KMyListBoxItem( const KMyListBoxItem* item );
        KMyListBoxItem( const KURL&, bool );

        static void setName( bool b );
        static void setPreview( bool b );
        void setPixmap( const QPixmap & pix );

        static bool preview() { return m_preview; }
        static bool name() { return m_name; }

        bool dir() const { return m_dir; }
        KURL url() const { return m_url; }
        const QPixmap* pixmap() const { return &pm; }

    private:
        QString text() const;

    protected:
        virtual void paint( QPainter *painter );
        virtual int width( const QListBox* ) const;
        virtual int height( const QListBox* lb ) const;

    private:
        KURL m_url;
        bool m_dir;
        QPixmap pm;

        static bool m_preview;
        static bool m_name;
};

#endif
